exception Unimplemented
exception InvalidResult

open Core
open Integer.Expression

let _ =
    let lexbuf = Lexing.from_channel In_channel.stdin in
        Out_channel.output_string stdout "import java.util.List;";
        Out_channel.newline stdout;
        Out_channel.output_string stdout "import java.util.LinkedList;";
        Out_channel.newline stdout;
        Out_channel.output_string stdout "public class Main {";
        Out_channel.newline stdout;
        Out_channel.output_string stdout "    public static void main(String[] args) {";
        Out_channel.newline stdout;
        let rec loop type_env var_idx =
            try
                let parsed = Parser.main Lexer.token lexbuf in match parsed with 
                    | Expression expr -> loop type_env var_idx
                    | Statement stmt -> match stmt with 
                        | Assign(v1, expr) -> 
                            let (v2, var_idx) = emit_expression type_env var_idx expr in 
                            emit_declaration v1 type_env expr;
                            let () = emit_assignment v1 v2 in 
                                loop (Map.set type_env ~key:v1 ~data:(type_expression type_env expr)) var_idx
                        | Print(expr) -> 
                            Out_channel.output_string stdout "        ";
                            Out_channel.output_string stdout ("System.out.println(" ^ (string_of_expression expr) ^ ");\n"); 
                            loop type_env var_idx
                        (*| _ -> raise Unimplemented *)
            with Lexer.Eof ->
                Out_channel.output_string stdout ("    }");
                Out_channel.newline stdout;
                Out_channel.output_string stdout ("}");
                Out_channel.newline stdout;
        in loop String.Map.empty 0
;;

(*open OUnit *)

(*
let tests = "integer eval tests">::: [
    "number" >:: (fun _ -> assert_equal 1 (IntegerExpression.eval [] (IntegerExpression.Num 1)));
    "variable" >:: (fun _ -> assert_equal 1 (IntegerExpression.eval [("x", 1)] (IntegerExpression.Var "x")));
    "one"   >:: (fun _ -> assert_equal [1] (One.rev [1]));
    "two"   >:: (fun _ -> assert_equal [2; 1] (One.rev [1; 2]));
    "three" >:: (fun _ -> assert_equal [3; 2; 1] (One.rev [1; 2; 3]));
]
let _ = run_test_tt_main tests
*)
