%token <int> INT
%token <string> VAR 
%token PLUS MINUS TIMES DIV POWER
%token LBRACKET RBRACKET COMMA IN IS
%token EQUALS
%token TRUE FALSE
%token LPAREN RPAREN
%token NEWLINE INDENT DEDENT WS 
%token AND OR NOT
%token IF ELSE FOR
%token PRINT
%left EQUALS
%left PLUS MINUS        /* lowest precedence */
%left TIMES DIV         /* medium precedence */
%right POWER            /* medium precedence */
%left OR
%left AND
%left NOT
%nonassoc UMINUS        /* highest precedence */
%right IF
%right ELSE
%start main             /* the entry point */
%type <Integer.Expression.program> main
%%

main:
    expr NEWLINE            { Expression($1) }
  | simple_stmt NEWLINE     { Statement($1) }
;
simple_stmt:
    VAR EQUALS expr           { Assign($1, $3) }
  | PRINT LPAREN expr RPAREN  { Print($3) }
;

expr: comprehension_expr { $1 }

comprehension_expr:
    LBRACKET expr comp_for RBRACKET { ListComprehension($2, $3) } 
  | if_expr { $1 }

comp_iter: 
      comp_for { $1 }
    | comp_if  { $1 }

comp_for: 
      FOR VAR IN test { ComprehensionFor($2, $4, End) } 
    | FOR VAR IN test comp_iter { ComprehensionFor($2, $4, $5) } 

comp_if:
      IF test { ComprehensionIf($2, End) }
    | IF test comp_iter { ComprehensionIf($2, $3) }

if_expr:
    misc_expr IF test ELSE if_expr { IfElse($3, $1, $5) } 
  | misc_expr { $1 } 
;
misc_expr:
    INT                     { Num $1  }
  | TRUE                    { Boolean true }
  | FALSE                   { Boolean false }
  | VAR                     { Var $1  }
  | LPAREN expr RPAREN   { $2 }
  | misc_expr PLUS misc_expr          { BinOp(Plus, $1, $3) }
  | misc_expr POWER misc_expr         { BinOp(Power, $1, $3) }
  | misc_expr MINUS misc_expr         { BinOp(Minus, $1, $3) }
  | misc_expr TIMES misc_expr         { BinOp(Times, $1, $3) }
  | misc_expr DIV misc_expr           { BinOp(Divide, $1, $3) }
  | list_expr { $1 }
  | MINUS misc_expr %prec UMINUS { match $2 with 
    | Num(x) -> Num(-x)
    | y -> UnaryOp(Negation, y)
  }
;

list_expr:
  | LBRACKET RBRACKET { List([]) }
  | LBRACKET inner_list_expr RBRACKET { $2 }

inner_list_expr:
      expr { List([$1]) }
    | inner_list_expr COMMA expr { List(List.append (Integer.Expression.unwrap_list $1) [$3]) }
    
test: 
      and_test           { $1 }
    | and_test OR test   { Or($1, $3) }
;
and_test: 
      not_test             { $1}
    | not_test AND and_test  { And($1, $3) }
;
not_test: 
      comparison           { $1 }
    | NOT not_test       { Not($2) }  
;

comparison: 
      misc_expr                { $1 }
    | misc_expr comp_op misc_expr   { Comparison($2, $1, $3) } 
;

comp_op: 
     EQUALS EQUALS { Equal }
    |'<'  { LessThan } 
    |'!=' { NotEqual }
;
